@@ -11,7 +11,7 @@
 #endif
 
 #include "batched_cg_policy.h"
-OSL_NAMESPACE_ENTER
+OSL_NAMESPACE_BEGIN
 namespace __OSL_WIDE_PVT {
 
 namespace {
@@ -29,7 +29,7 @@ struct BatchedCGPolicy<Param::WV, Param::WV, Param::WF, Param::WV, Param::WF> {
 }  // namespace
 
 }  // namespace __OSL_WIDE_PVT
-OSL_NAMESPACE_EXIT
+OSL_NAMESPACE_END
 
 // To improve parallel compile times, split noise with float results and
 // Vec3 results into different cpp files
