@@ -12,7 +12,7 @@
 
 #include "oslexec_pvt.h"
 
-OSL_NAMESPACE_ENTER
+OSL_NAMESPACE_BEGIN
 namespace __OSL_WIDE_PVT {
 
 OSL_USING_DATA_WIDTH(__OSL_WIDTH)
@@ -29,9 +29,9 @@ using WidthTag = OSL::WidthOf<__OSL_WIDTH>;
 // firstcheck,nchecks are used to check just one element of an array.
 OSL_BATCHOP void
 __OSL_OP(naninf_check)(int ncomps, const void* vals_, int has_derivs,
-                       void* bsg_, const void* sourcefile, int sourceline,
-                       void* symbolname, int firstcheck, int nchecks,
-                       const void* opname)
+                       void* bsg_, ustring_pod sourcefile, int sourceline,
+                       ustring_pod symbolname, int firstcheck, int nchecks,
+                       ustring_pod opname)
 {
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
     ShadingContext* ctx = bsg->uniform.context;
@@ -39,7 +39,7 @@ __OSL_OP(naninf_check)(int ncomps, const void* vals_, int has_derivs,
     for (int d = 0; d < (has_derivs ? 3 : 1); ++d) {
         for (int c = firstcheck, e = c + nchecks; c < e; ++c) {
             int i = d * ncomps + c;
-            if (!OIIO::isfinite(vals[i])) {
+            if (!std::isfinite(vals[i])) {
                 ctx->errorfmt("Detected {} value in {}{} at {}:{} (op {})",
                               vals[i], d > 0 ? "the derivatives of " : "",
                               USTR(symbolname), USTR(sourcefile), sourceline,
@@ -56,9 +56,9 @@ __OSL_OP(naninf_check)(int ncomps, const void* vals_, int has_derivs,
 OSL_BATCHOP void
 __OSL_MASKED_OP1(naninf_check_offset,
                  i)(int mask_value, int ncomps, const void* vals_,
-                    int has_derivs, void* bsg_, const void* sourcefile,
-                    int sourceline, void* symbolname, int firstcheck,
-                    int nchecks, const void* opname)
+                    int has_derivs, void* bsg_, ustring_pod sourcefile,
+                    int sourceline, ustring_pod symbolname, int firstcheck,
+                    int nchecks, ustring_pod opname)
 {
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
     ShadingContext* ctx = bsg->uniform.context;
@@ -68,12 +68,13 @@ __OSL_MASKED_OP1(naninf_check_offset,
         for (int c = firstcheck, e = c + nchecks; c < e; ++c) {
             int i = d * ncomps + c;
             mask.foreach ([=](ActiveLane lane) -> void {
-                if (!OIIO::isfinite(vals[i * __OSL_WIDTH + lane])) {
+                if (!std::isfinite(vals[i * __OSL_WIDTH + lane])) {
                     ctx->errorfmt(
                         "Detected {} value in {}{} at {}:{} (op {}) batch lane:{}",
                         vals[i * __OSL_WIDTH + lane],
                         d > 0 ? "the derivatives of " : "", USTR(symbolname),
-                        USTR(sourcefile), sourceline, USTR(opname), lane);
+                        USTR(sourcefile), sourceline, USTR(opname),
+                        lane.value());
                     // continue checking all data lanes, and all components
                     // for that matter, we want to find all issues, not just
                     // the 1st, right?
@@ -88,12 +89,12 @@ __OSL_MASKED_OP1(naninf_check_offset,
 
 // Wide vals + mask + varying index
 OSL_BATCHOP void
-__OSL_MASKED_OP1(naninf_check_offset, Wi)(int mask_value, int ncomps,
-                                          const void* vals_, int has_derivs,
-                                          void* bsg_, const void* sourcefile,
-                                          int sourceline, void* symbolname,
-                                          const void* wide_offsets_ptr,
-                                          int nchecks, const void* opname)
+__OSL_MASKED_OP1(naninf_check_offset,
+                 Wi)(int mask_value, int ncomps, const void* vals_,
+                     int has_derivs, void* bsg_, ustring_pod sourcefile,
+                     int sourceline, ustring_pod symbolname,
+                     const void* wide_offsets_ptr, int nchecks,
+                     ustring_pod opname)
 {
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
     ShadingContext* ctx = bsg->uniform.context;
@@ -106,12 +107,13 @@ __OSL_MASKED_OP1(naninf_check_offset, Wi)(int mask_value, int ncomps,
             int firstcheck = wOffsets[lane];
             for (int c = firstcheck, e = c + nchecks; c < e; ++c) {
                 int i = d * ncomps + c;
-                if (!OIIO::isfinite(vals[i * __OSL_WIDTH + lane])) {
+                if (!std::isfinite(vals[i * __OSL_WIDTH + lane])) {
                     ctx->errorfmt(
                         "Detected {} value in {}{} at {}:{} (op {}) batch lane:{}",
                         vals[i * __OSL_WIDTH + lane],
                         d > 0 ? "the derivatives of " : "", USTR(symbolname),
-                        USTR(sourcefile), sourceline, USTR(opname), lane);
+                        USTR(sourcefile), sourceline, USTR(opname),
+                        lane.value());
                     // continue checking all data lanes, and all components
                     // for that matter, we want to find all issues, not just
                     // the 1st, right?
@@ -129,9 +131,9 @@ __OSL_MASKED_OP1(naninf_check_offset, Wi)(int mask_value, int ncomps,
 OSL_BATCHOP void
 __OSL_OP2(uninit_check_values_offset, X,
           i)(long long typedesc_, void* vals_, void* bsg_,
-             const void* sourcefile, int sourceline, const char* groupname,
-             int layer, const char* layername, const char* shadername,
-             int opnum, const char* opname, int argnum, void* symbolname,
+             ustring_pod sourcefile, int sourceline, ustring_pod groupname_,
+             int layer, ustring_pod layername_, ustring_pod shadername,
+             int opnum, ustring_pod opname, int argnum, ustring_pod symbolname,
              int firstcheck, int nchecks)
 {
     TypeDesc typedesc   = TYPEDESC(typedesc_);
@@ -141,7 +143,7 @@ __OSL_OP2(uninit_check_values_offset, X,
     if (typedesc.basetype == TypeDesc::FLOAT) {
         float* vals = (float*)vals_;
         for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c)
-            if (!OIIO::isfinite(vals[c])) {
+            if (!std::isfinite(vals[c])) {
                 uninit  = true;
                 vals[c] = 0;
             }
@@ -163,12 +165,14 @@ __OSL_OP2(uninit_check_values_offset, X,
             }
     }
     if (uninit) {
+        ustring groupname = USTR(groupname_);
+        ustring layername = USTR(layername_);
         ctx->errorfmt(
             "Detected possible use of uninitialized value in {} {} at {}:{} (group {}, layer {} {}, shader {}, op {} '{}', arg {})",
             typedesc, USTR(symbolname), USTR(sourcefile), sourceline,
-            (groupname && groupname[0]) ? groupname : "<unnamed group>", layer,
-            (layername && layername[0]) ? layername : "<unnamed layer>",
-            shadername, opnum, USTR(opname), argnum);
+            groupname.empty() ? "<unnamed group>" : groupname.c_str(), layer,
+            layername.empty() ? "<unnamed layer>" : layername.c_str(),
+            USTR(shadername), opnum, USTR(opname), argnum);
     }
 }
 
@@ -179,10 +183,11 @@ __OSL_OP2(uninit_check_values_offset, X,
 OSL_BATCHOP void
 __OSL_MASKED_OP2(uninit_check_values_offset, WX,
                  i)(int mask_value, long long typedesc_, void* vals_,
-                    void* bsg_, const void* sourcefile, int sourceline,
-                    const char* groupname, int layer, const char* layername,
-                    const char* shadername, int opnum, const char* opname,
-                    int argnum, void* symbolname, int firstcheck, int nchecks)
+                    void* bsg_, ustring_pod sourcefile, int sourceline,
+                    ustring_pod groupname_, int layer, ustring_pod layername_,
+                    ustring_pod shadername, int opnum, ustring_pod opname,
+                    int argnum, ustring_pod symbolname, int firstcheck,
+                    int nchecks)
 {
     TypeDesc typedesc   = TYPEDESC(typedesc_);
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
@@ -195,7 +200,7 @@ __OSL_MASKED_OP2(uninit_check_values_offset, WX,
         float* vals = (float*)vals_;
         for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c)
             mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
-                if (!OIIO::isfinite(vals[c * __OSL_WIDTH + lane])) {
+                if (!std::isfinite(vals[c * __OSL_WIDTH + lane])) {
                     lanes_uninit.set_on(lane);
                     vals[c * __OSL_WIDTH + lane] = 0;
                 }
@@ -224,12 +229,86 @@ __OSL_MASKED_OP2(uninit_check_values_offset, WX,
             });
     }
     if (lanes_uninit.any_on()) {
+        ustring groupname = USTR(groupname_);
+        ustring layername = USTR(layername_);
         ctx->errorfmt(
             "Detected possible use of uninitialized value in {} {} at {}:{} (group {}, layer {} {}, shader {}, op {} '{}', arg {}) for lanes({:x}) of batch",
             typedesc, USTR(symbolname), USTR(sourcefile), sourceline,
-            (groupname && groupname[0]) ? groupname : "<unnamed group>", layer,
-            (layername && layername[0]) ? layername : "<unnamed layer>",
-            shadername, opnum, USTR(opname), argnum, lanes_uninit.value());
+            groupname.empty() ? "<unnamed group>" : groupname.c_str(), layer,
+            layername.empty() ? "<unnamed layer>" : layername.c_str(),
+            USTR(shadername), opnum, USTR(opname), argnum,
+            lanes_uninit.value());
+    }
+}
+
+
+
+// Many parameters, but the 3 parameters used in the function name
+// correspond to:  "vals", "firstcheck", and "nchecks"
+OSL_BATCHOP void
+__OSL_MASKED_OP3(uninit_check_values_offset, WX, i,
+                 Wi)(int mask_value, long long typedesc_, void* vals_,
+                     void* bsg_, ustring_pod sourcefile, int sourceline,
+                     ustring_pod groupname_, int layer, ustring_pod layername_,
+                     ustring_pod shadername, int opnum, ustring_pod opname,
+                     int argnum, ustring_pod symbolname, int firstcheck,
+                     int* nchecks_)
+{
+    TypeDesc typedesc   = TYPEDESC(typedesc_);
+    auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
+    ShadingContext* ctx = bsg->uniform.context;
+    const Mask mask(mask_value);
+
+    Mask lanes_uninit(false);
+
+    if (typedesc.basetype == TypeDesc::FLOAT) {
+        float* vals = (float*)vals_;
+        mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
+            int nchecks = nchecks_[lane];
+            for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c) {
+                if (!std::isfinite(vals[c * __OSL_WIDTH + lane])) {
+                    lanes_uninit.set_on(lane);
+                    vals[c * __OSL_WIDTH + lane] = 0;
+                }
+            }
+        });
+    }
+    if (typedesc.basetype == TypeDesc::INT) {
+        int* vals = (int*)vals_;
+        mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
+            int nchecks = nchecks_[lane];
+            for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c) {
+                if (vals[c * __OSL_WIDTH + lane]
+                    == std::numeric_limits<int>::min()) {
+                    lanes_uninit.set_on(lane);
+                    vals[c * __OSL_WIDTH + lane] = 0;
+                }
+            }
+        });
+    }
+    if (typedesc.basetype == TypeDesc::STRING) {
+        ustring* vals = (ustring*)vals_;
+        mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
+            int nchecks = nchecks_[lane];
+            for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c) {
+                if (vals[c * __OSL_WIDTH + lane]
+                    == Strings::uninitialized_string) {
+                    lanes_uninit.set_on(lane);
+                    vals[c * __OSL_WIDTH + lane] = ustring();
+                }
+            }
+        });
+    }
+    if (lanes_uninit.any_on()) {
+        ustring groupname = USTR(groupname_);
+        ustring layername = USTR(layername_);
+        ctx->errorfmt(
+            "Detected possible use of uninitialized value in {} {} at {}:{} (group {}, layer {} {}, shader {}, op {} '{}', arg {}) for lanes({:x}) of batch",
+            typedesc, USTR(symbolname), USTR(sourcefile), sourceline,
+            groupname.empty() ? "<unnamed group>" : groupname.c_str(), layer,
+            layername.empty() ? "<unnamed layer>" : layername.c_str(),
+            USTR(shadername), opnum, USTR(opname), argnum,
+            lanes_uninit.value());
     }
 }
 
@@ -240,11 +319,11 @@ __OSL_MASKED_OP2(uninit_check_values_offset, WX,
 OSL_BATCHOP void
 __OSL_MASKED_OP2(uninit_check_values_offset, X,
                  Wi)(int mask_value, long long typedesc_, void* vals_,
-                     void* bsg_, const void* sourcefile, int sourceline,
-                     const char* groupname, int layer, const char* layername,
-                     const char* shadername, int opnum, const char* opname,
-                     int argnum, void* symbolname, const void* wide_offsets_ptr,
-                     int nchecks)
+                     void* bsg_, ustring_pod sourcefile, int sourceline,
+                     ustring_pod groupname_, int layer, ustring_pod layername_,
+                     ustring_pod shadername, int opnum, ustring_pod opname,
+                     int argnum, ustring_pod symbolname,
+                     const void* wide_offsets_ptr, int nchecks)
 {
     TypeDesc typedesc   = TYPEDESC(typedesc_);
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
@@ -258,7 +337,7 @@ __OSL_MASKED_OP2(uninit_check_values_offset, X,
         mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
             int firstcheck = wOffsets[lane];
             for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c)
-                if (!OIIO::isfinite(vals[c])) {
+                if (!std::isfinite(vals[c])) {
                     lanes_uninit.set_on(lane);
                     vals[c] = 0;
                 }
@@ -288,12 +367,15 @@ __OSL_MASKED_OP2(uninit_check_values_offset, X,
     }
 
     if (lanes_uninit.any_on()) {
+        ustring groupname = USTR(groupname_);
+        ustring layername = USTR(layername_);
         ctx->errorfmt(
             "Detected possible use of uninitialized value in {} {} at {}:{} (group {}, layer {} {}, shader {}, op {} '{}', arg {}) for lanes({:x}) of batch",
             typedesc, USTR(symbolname), USTR(sourcefile), sourceline,
-            (groupname && groupname[0]) ? groupname : "<unnamed group>", layer,
-            (layername && layername[0]) ? layername : "<unnamed layer>",
-            shadername, opnum, USTR(opname), argnum, lanes_uninit.value());
+            groupname.empty() ? "<unnamed group>" : groupname.c_str(), layer,
+            layername.empty() ? "<unnamed layer>" : layername.c_str(),
+            USTR(shadername), opnum, USTR(opname), argnum,
+            lanes_uninit.value());
     }
 }
 
@@ -304,11 +386,11 @@ __OSL_MASKED_OP2(uninit_check_values_offset, X,
 OSL_BATCHOP void
 __OSL_MASKED_OP2(uninit_check_values_offset, WX,
                  Wi)(int mask_value, long long typedesc_, void* vals_,
-                     void* bsg_, const void* sourcefile, int sourceline,
-                     const char* groupname, int layer, const char* layername,
-                     const char* shadername, int opnum, const char* opname,
-                     int argnum, void* symbolname, const void* wide_offsets_ptr,
-                     int nchecks)
+                     void* bsg_, ustring_pod sourcefile, int sourceline,
+                     ustring_pod groupname_, int layer, ustring_pod layername_,
+                     ustring_pod shadername, int opnum, ustring_pod opname,
+                     int argnum, ustring_pod symbolname,
+                     const void* wide_offsets_ptr, int nchecks)
 {
     TypeDesc typedesc   = TYPEDESC(typedesc_);
     auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
@@ -322,7 +404,7 @@ __OSL_MASKED_OP2(uninit_check_values_offset, WX,
         mask.foreach ([=, &lanes_uninit](ActiveLane lane) -> void {
             int firstcheck = wOffsets[lane];
             for (int c = firstcheck, e = firstcheck + nchecks; c < e; ++c)
-                if (!OIIO::isfinite(vals[c * __OSL_WIDTH + lane])) {
+                if (!std::isfinite(vals[c * __OSL_WIDTH + lane])) {
                     lanes_uninit.set_on(lane);
                     vals[c * __OSL_WIDTH + lane] = 0;
                 }
@@ -354,33 +436,39 @@ __OSL_MASKED_OP2(uninit_check_values_offset, WX,
     }
 
     if (lanes_uninit.any_on()) {
+        ustring groupname = USTR(groupname_);
+        ustring layername = USTR(layername_);
         ctx->errorfmt(
             "Detected possible use of uninitialized value in {} {} at {}:{} (group {}, layer {} {}, shader {}, op {} '{}', arg {}) for lanes({:x}) of batch",
             typedesc, USTR(symbolname), USTR(sourcefile), sourceline,
-            (groupname && groupname[0]) ? groupname : "<unnamed group>", layer,
-            (layername && layername[0]) ? layername : "<unnamed layer>",
-            shadername, opnum, USTR(opname), argnum, lanes_uninit.value());
+            groupname.empty() ? "<unnamed group>" : groupname.c_str(), layer,
+            layername.empty() ? "<unnamed layer>" : layername.c_str(),
+            USTR(shadername), opnum, USTR(opname), argnum,
+            lanes_uninit.value());
     }
 }
 
 
 
 OSL_BATCHOP int
-__OSL_OP(range_check)(int indexvalue, int length, const char* symname,
-                      void* bsg_, const void* sourcefile, int sourceline,
-                      const char* groupname, int layer, const char* layername,
-                      const char* shadername)
+__OSL_OP(range_check)(int indexvalue, int length, ustring_pod symname,
+                      void* bsg_, ustring_pod sourcefile, int sourceline,
+                      ustring_pod groupname_, int layer, ustring_pod layername_,
+                      ustring_pod shadername)
 {
     if (indexvalue < 0 || indexvalue >= length) {
+        ustring groupname   = USTR(groupname_);
+        ustring layername   = USTR(layername_);
         auto* bsg           = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
         ShadingContext* ctx = bsg->uniform.context;
-        ctx->errorfmt(
-            "Index [{}] out of range {}[0..{}]: {}:{}"
-            " (group {}, layer {} {}, shader {})",
-            indexvalue, USTR(symname), length - 1, USTR(sourcefile), sourceline,
-            (groupname && groupname[0]) ? groupname : "<unnamed group>", layer,
-            (layername && layername[0]) ? layername : "<unnamed layer>",
-            USTR(shadername));
+        ctx->errorfmt("Index [{}] out of range {}[0..{}]: {}:{}"
+                      " (group {}, layer {} {}, shader {})",
+                      indexvalue, USTR(symname), length - 1, USTR(sourcefile),
+                      sourceline,
+                      groupname.empty() ? "<unnamed group>" : groupname.c_str(),
+                      layer,
+                      layername.empty() ? "<unnamed layer>" : layername.c_str(),
+                      USTR(shadername));
         if (indexvalue >= length)
             indexvalue = length - 1;
         else
@@ -393,12 +481,14 @@ __OSL_OP(range_check)(int indexvalue, int length, const char* symname,
 
 OSL_BATCHOP void
 __OSL_MASKED_OP(range_check)(void* wide_indexvalue, unsigned int mask_value,
-                             int length, const char* symname, void* bsg_,
-                             const void* sourcefile, int sourceline,
-                             const char* groupname, int layer,
-                             const char* layername, const char* shadername)
+                             int length, ustring_pod symname, void* bsg_,
+                             ustring_pod sourcefile, int sourceline,
+                             ustring_pod groupname_, int layer,
+                             ustring_pod layername_, ustring_pod shadername)
 {
-    auto* bsg = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
+    ustring groupname = USTR(groupname_);
+    ustring layername = USTR(layername_);
+    auto* bsg         = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
     Masked<int> wIndexValue(wide_indexvalue, Mask(mask_value));
     wIndexValue.mask().foreach ([=](ActiveLane lane) -> void {
         int indexvalue = wIndexValue[lane];
@@ -408,9 +498,9 @@ __OSL_MASKED_OP(range_check)(void* wide_indexvalue, unsigned int mask_value,
                 "Index [{}] out of range {}[0..{}]: {}:{} (group {}, layer {} {}, shader {})",
                 indexvalue, USTR(symname), length - 1, USTR(sourcefile),
                 sourceline,
-                (groupname && groupname[0]) ? groupname : "<unnamed group>",
+                groupname.empty() ? "<unnamed group>" : groupname.c_str(),
                 layer,
-                (layername && layername[0]) ? layername : "<unnamed layer>",
+                layername.empty() ? "<unnamed layer>" : layername.c_str(),
                 USTR(shadername));
             if (indexvalue >= length)
                 indexvalue = length - 1;
@@ -425,17 +515,17 @@ __OSL_MASKED_OP(range_check)(void* wide_indexvalue, unsigned int mask_value,
 
 
 OSL_BATCHOP int
-__OSL_OP1(get_attribute, s)(void* bsg_, int dest_derivs, void* obj_name_,
-                            void* attr_name_, int array_lookup, int index,
+__OSL_OP1(get_attribute, s)(void* bsg_, int dest_derivs, ustring_pod obj_name_,
+                            ustring_pod attr_name_, int array_lookup, int index,
                             const void* attr_type, void* wide_attr_dest,
                             int mask_)
 {
     Mask mask(mask_);
-    ASSERT(mask.any_on());
+    OSL_ASSERT(mask.any_on());
 
-    auto* bsg                = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
-    const ustring& obj_name  = USTR(obj_name_);
-    const ustring& attr_name = USTR(attr_name_);
+    auto* bsg         = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
+    ustring obj_name  = USTR(obj_name_);
+    ustring attr_name = USTR(attr_name_);
 
     // Ignoring m_next_failed_attrib cache for now,
     // might be faster
@@ -457,15 +547,15 @@ __OSL_OP1(get_attribute, s)(void* bsg_, int dest_derivs, void* obj_name_,
 
 OSL_BATCHOP int
 __OSL_MASKED_OP1(get_attribute,
-                 Ws)(void* bsg_, int dest_derivs, void* obj_name_,
-                     void* wattr_name_, int array_lookup, int index,
+                 Ws)(void* bsg_, int dest_derivs, ustring_pod obj_name_,
+                     ustring_pod* wattr_name_, int array_lookup, int index,
                      const void* attr_type, void* wide_attr_dest, int mask_)
 {
     Mask mask(mask_);
-    ASSERT(mask.any_on());
+    OSL_ASSERT(mask.any_on());
 
-    auto* bsg               = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
-    const ustring& obj_name = USTR(obj_name_);
+    auto* bsg        = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
+    ustring obj_name = USTR(obj_name_);
     Wide<const ustring> wAttrName(wattr_name_);
     auto* renderer = bsg->uniform.context->batched<__OSL_WIDTH>().renderer();
 
@@ -504,13 +594,14 @@ __OSL_MASKED_OP1(get_attribute,
 
 
 OSL_BATCHOP bool
-__OSL_OP(get_attribute_uniform)(void* bsg_, int dest_derivs, void* obj_name_,
-                                void* attr_name_, int array_lookup, int index,
+__OSL_OP(get_attribute_uniform)(void* bsg_, int dest_derivs,
+                                ustring_pod obj_name_, ustring_pod attr_name_,
+                                int array_lookup, int index,
                                 const void* attr_type, void* attr_dest)
 {
-    auto* bsg                = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
-    const ustring& obj_name  = USTR(obj_name_);
-    const ustring& attr_name = USTR(attr_name_);
+    auto* bsg         = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
+    ustring obj_name  = USTR(obj_name_);
+    ustring attr_name = USTR(attr_name_);
 
     auto* renderer = bsg->uniform.context->batched<__OSL_WIDTH>().renderer();
 
@@ -530,7 +621,7 @@ __OSL_OP(get_attribute_uniform)(void* bsg_, int dest_derivs, void* obj_name_,
 
 
 OSL_BATCHOP int
-__OSL_OP(bind_interpolated_param)(void* bsg_, const void* name, long long type,
+__OSL_OP(bind_interpolated_param)(void* bsg_, ustring_pod name, long long type,
                                   int userdata_has_derivs, void* userdata_data,
                                   int symbol_has_derivs, void* symbol_data,
                                   int symbol_data_size,
@@ -580,7 +671,7 @@ __OSL_OP(raytype_bit)(void* bsg_, int bit)
 
 // Asked if the raytype is a name we can't know until mid-shader.
 OSL_BATCHOP int
-__OSL_OP(raytype_name)(void* bsg_, void* name)
+__OSL_OP(raytype_name)(void* bsg_, ustring_pod name)
 {
     auto* bsg = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
     int bit   = bsg->uniform.context->shadingsys().raytype_bit(USTR(name));
@@ -590,7 +681,7 @@ __OSL_OP(raytype_name)(void* bsg_, void* name)
 
 
 OSL_BATCHOP void
-__OSL_MASKED_OP(raytype_name)(void* bsg_, void* r_, void* name_,
+__OSL_MASKED_OP(raytype_name)(void* bsg_, void* r_, ustring* name_,
                               unsigned int mask_value)
 {
     auto* bsg = reinterpret_cast<BatchedShaderGlobals*>(bsg_);
@@ -606,6 +697,6 @@ __OSL_MASKED_OP(raytype_name)(void* bsg_, void* r_, void* name_,
 }
 
 }  // namespace __OSL_WIDE_PVT
-OSL_NAMESPACE_EXIT
+OSL_NAMESPACE_END
 
 #include "undef_opname_macros.h"
